/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package grafica;
import java.awt.*;
import java.awt.geom.AffineTransform;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import java.util.Vector;
import javax.swing.*;
import struttura.Corsia;
import struttura.Flusso;
import struttura.Incrocio;
import struttura.ListaFlussi;
import struttura.Strada;


/**
 *
 * @author Fabio
 */
public class Disegno extends JPanel {
    
    public Disegno() {
        this.adaptiveScale = true;
        this.puntiCorsie = new Vector();
        this.puntiAttraversamenti = new Vector();
        this.flussiVerdi = new ListaFlussi();
    }
    
    public Disegno(ListaFlussi verdi) {
        this.adaptiveScale = true;
        this.puntiCorsie = new Vector();
        this.puntiAttraversamenti = new Vector();
        this.flussiVerdi = verdi;
    }
    
    public Disegno(double scale) {
        this.adaptiveScale = false;
        this.scale = scale;
        this.puntiCorsie = new Vector();
        this.puntiAttraversamenti = new Vector();
        this.flussiVerdi = new ListaFlussi();
    }
    
    public Disegno(double scale, ListaFlussi verdi) {
        this.adaptiveScale = false;
        this.scale = scale;
        this.puntiCorsie = new Vector();
        this.puntiAttraversamenti = new Vector();
        this.flussiVerdi = verdi;
    }
    
    public void addPuntoCorsia(int numCorsia, Point2D punto, AffineTransform trasformazione, double angolo) {
        Vector data = new Vector();
        data.add(numCorsia);
        data.add(punto);
        data.add(trasformazione);
        data.add(angolo);
        puntiCorsie.add(data);
    }
    
    public void addPuntoAttraversamento(int numAttraversamento, Point2D punto, AffineTransform trasformazione) {
        Vector data = new Vector();
        data.add(numAttraversamento);
        data.add(punto);
        data.add(trasformazione);
        puntiAttraversamenti.add(data);
    }
    
    public void setListaFlussiVerdi(ListaFlussi verdi) {
        this.flussiVerdi = verdi;
    }
    
    @Override
    public void paintComponent(Graphics g){       
        Graphics2D g2 = (Graphics2D) g;
        g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
        //Mi recupero le coordinate del centro del pannello
        Dimension size = getSize();
        int x = (size.width)/2;
        int y = (size.height)/2;
        // traslo tutto di x e y
        g2.translate(x, y);
        // se scale è uguale a -1, allora lo calcolo automaticamente
        if (adaptiveScale)
            scale = (double) Math.min(size.width, size.height) / LATO_DISEGNO;
        // scalo tutto di un fattore scale
        g2.scale(scale, scale);
        //Mi salvo la matrice originaria di trasformazione
        AffineTransform backup = g2.getTransform();
        // azzero la struttura puntiCorsie
        puntiCorsie = new Vector();
         //Mi recupero la costante PI
        double pi = 2 * Math.PI;
        //Vado a recupare la struttura dati dell'incrocio
        Incrocio incrocio = Incrocio.getInstance();
        int numeroStrade = incrocio.getListaStrade().size();
        Vector<Strada> listaStrade = incrocio.getListaStrade();
        //Mi calcolo l'angolo alfa di rotazione, sul quale effettuo le rotazioni
        double alfa = 0;
        int max = 0;
        int numeroCorsie = 0;
        int maxCorsie = 0;
        for (int i = 0; i < numeroStrade; i++){
            Strada temp = (Strada)listaStrade.elementAt(i);
            int numeroEntranti = temp.getListaCorsieEntranti().size();
            int numeroUscenti = temp.getListaCorsieUscenti().size();
            numeroCorsie = numeroEntranti + numeroUscenti + numeroCorsie;
            max = Math.max(max, numeroEntranti + numeroUscenti);
            maxCorsie = Math.max(maxCorsie, numeroEntranti + numeroUscenti);
        }        
        //L'angolo adesso viene calcolato in base al numero delle corsie, non più delle strade
        double angolo = pi / numeroCorsie;
        double spicchio = angolo * maxCorsie;
        double raggio = 0.0d;
        if (numeroCorsie != maxCorsie)            
            raggio = (maxCorsie * StradaForma.LARGHEZZA / 2) / Math.cos((Math.PI - spicchio)/2);
        int raggioIntero = (int)Math.round(raggio);
        for (int i = 0; i < numeroStrade; i++){
            Strada temp = (Strada)listaStrade.elementAt(i);
            int numeroEntranti = temp.getListaCorsieEntranti().size();
            int numeroUscenti = temp.getListaCorsieUscenti().size();
            if (i > 0)
                alfa += angolo * (numeroEntranti + numeroUscenti) /2 ;
            StradaForma strada = new StradaForma(temp, -(numeroEntranti + numeroUscenti)* StradaForma.LARGHEZZA / 2, 0, raggioIntero, i, this, alfa);            
            g2.rotate(alfa);
            strada.draw(g2);
            g2.setTransform(backup);
            alfa += angolo * (numeroEntranti + numeroUscenti) / 2;
            Strada succTemp;
            if (i == numeroStrade - 1)
                succTemp = (Strada) listaStrade.elementAt(0);
            else
                succTemp = (Strada) listaStrade.elementAt(i + 1);
            if (temp.getAttraversamento() || succTemp.getAttraversamento()) {
                int index = -1;
                if (numeroUscenti > 0)
                    index = ((Corsia) temp.getListaCorsieUscenti().lastElement()).getIndice();
                else
                    if (numeroEntranti > 0)
                        index = ((Corsia) temp.getListaCorsieEntranti().lastElement()).getIndice();
                if (index != -1) {
                    g2.setColor(Color.black);
                    g2.rotate(alfa);
                    Ellipse2D.Double attrav = new Ellipse2D.Double(0, raggio + StradaForma.ALTEZZA_STRISCE + StradaForma.STRISCE, 3, 3);
                    g2.draw(attrav);
                    AffineTransform trasf = new AffineTransform();
                    trasf.setToRotation(alfa);
                    addPuntoAttraversamento(index + 1, new Point2D.Double(0, raggio + StradaForma.ALTEZZA_STRISCE + StradaForma.STRISCE), trasf);
                    String numeroAttravStr = (index + 1) + "";
                    g2.drawString(numeroAttravStr,  -numeroAttravStr.length() * 4, raggioIntero + StradaForma.ALTEZZA_STRISCE + StradaForma.STRISCE * 6);
                    g2.setTransform(backup);
                }
            }
        }
        //Disegno il cerchio
        g2.setColor(Color.gray);
        g2.setStroke(new BasicStroke(2));
        Ellipse2D.Double ellisse = new Ellipse2D.Double(-raggio, -raggio, raggio * 2, raggio * 2);
        g2.fill(ellisse);
        g2.draw(ellisse);
        if (!flussiVerdi.isEmpty()) {
            for (int i = 0; i < flussiVerdi.size(); i++) {
                Flusso flusso = flussiVerdi.getFlusso(i);
                int sorgente = flusso.getSorgente();
                int destinazione = flusso.getDestinazione();
                Vector rigaSorgente = null;
                Vector rigaDestinazione = null;
                for (int j = 0; j < puntiCorsie.size(); j++) {
                    Vector riga = (Vector) puntiCorsie.get(j);
                    int numCorsia = (Integer) riga.get(0);
                    if (numCorsia == sorgente)
                        rigaSorgente = riga;
                    if (numCorsia == destinazione)
                        rigaDestinazione = riga;
                }
                if (rigaSorgente != null && rigaDestinazione != null) {
                    Point2D.Double puntoSorg = (Point2D.Double) rigaSorgente.get(1);
                    AffineTransform trasfSorg = (AffineTransform) rigaSorgente.get(2);
                    puntoSorg = (Point2D.Double) trasfSorg.transform(puntoSorg, null);
                    double angoloSorg = (Double) rigaSorgente.get(3) + Math.PI / 2;
                    Point2D.Double puntoDest = (Point2D.Double) rigaDestinazione.get(1);
                    AffineTransform trasfDest = (AffineTransform) rigaDestinazione.get(2);
                    puntoDest = (Point2D.Double) trasfDest.transform(puntoDest, null);
                    double angoloDest = (Double) rigaDestinazione.get(3) + Math.PI / 2;
                    g2.setColor(Color.green);
                    g2.setTransform(backup);
                    JCurve curve = new JCurve(puntoSorg, angoloSorg, puntoDest, angoloDest, new Point2D.Double(0, 0), raggio);
                    curve.draw(g2);
                }
                rigaSorgente = null;
                rigaDestinazione = null;
                for (int j = 0; j < puntiAttraversamenti.size(); j++) {
                    Vector riga = (Vector) puntiAttraversamenti.get(j);
                    int numAttraversamento = (Integer) riga.get(0);
                    if (numAttraversamento == sorgente)
                        rigaSorgente = riga;
                    if (numAttraversamento == destinazione)
                        rigaDestinazione = riga;
                }
                if (rigaSorgente != null && rigaDestinazione != null) {
                    Point2D.Double puntoSorg = (Point2D.Double) rigaSorgente.get(1);
                    AffineTransform trasfSorg = (AffineTransform) rigaSorgente.get(2);
                    puntoSorg = (Point2D.Double) trasfSorg.transform(puntoSorg, null);
                    Point2D.Double puntoDest = (Point2D.Double) rigaDestinazione.get(1);
                    AffineTransform trasfDest = (AffineTransform) rigaDestinazione.get(2);
                    puntoDest = (Point2D.Double) trasfDest.transform(puntoDest, null);
                    g2.setColor(Color.green);
                    g2.setTransform(backup);
                    g2.draw(new Line2D.Double(puntoSorg, puntoDest));
                }
            }
        }
    }
    
    private static final int LATO_DISEGNO = StradaForma.LUNGHEZZA * 2 + StradaForma.ALTEZZA_STRISCE * 4;
    
    private boolean adaptiveScale;
    private double scale;
    private Vector puntiCorsie;
    private Vector puntiAttraversamenti;
    private ListaFlussi flussiVerdi;

}
