package grafica;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.text.*;

/**
 *	Gestore della finestra preposta all'inserimento di nuove ordinazioni.
 *	@see presentation.pac.PresentationActor
 */

	/**
	 *	Ogni oggetto di questa classe è... una chicca che potrebbe tornare
	 *		utile in molte applicazioni: si tratta di una casella di testo
	 *		che automaticamente bypassa l'inserimento di qualsiasi carattere
	 *		al suo interno, al di fuori dei caratteri numerici! Inoltre, non
	 *		consente l'inserimento di più di 3 cifre.
	 */	
public class NonPiuDiDueCifre extends JTextField implements ContaCaratteri {
    
    /** Indica quanti siano i caratteri massimi consentiti */
    private static final int MAX_CHARS_ADMITTED = 2;
    /**
    *	Costruttore-tipo delle <code>JTextField</code>
    */
    public NonPiuDiDueCifre() {
        super(MAX_CHARS_ADMITTED);
    }

    /**	Metodo di implementazione dell'interfaccia ContaCaratteri */
    public int quantiCaratteri() {
        return this.getText().length();
    }

    /**
    *	Attraverso l'overriding di questo metodo è possibile filtrare
    *		l'input con una classe creata ad arte, che estenda
    *		<code>PlainDocument</code>.
    *	@return Un oggetto di classe figlia di <code>PlainDocument</code>,
    *		che filtri l'input come voluto
    */
    protected Document createDefaultModel() {
        return new NumberDocument(this);
    }

    /**
    *	Ulteriore livello di <em>nested class</em>: questa classe è
    *		l'effettivo "filtro" per i caratteri digitati nella
    *		<code>JTextField</code>
    */
    private class NumberDocument extends PlainDocument {
        /** Il campo che sa contare i caratteri al proprio interno */
        private ContaCaratteri campoContaCaratteri;

        /**	Costruttore. */
        public NumberDocument(ContaCaratteri campo) {
            super();
            this.campoContaCaratteri = campo;
        }
        /**
        *	Attraverso l'overriding di questo metodo è possibile stabilire
        *		quali caratteri compariranno tra quelli digitati (dati
        *		volta per volta dal parametro stringa <code>str</code>)
        *		all'interno della casella di testo, e quanti.
        *	@param str Stringa digitata ad ogni pressione sulla tastiera
        *		(su cui effettuare il controllo per filtrare)
        */
        public void insertString(int offs, String str, AttributeSet a) throws BadLocationException {
            /*
            --------------------------------------------------------------------------------
            IL CONTROLLO DELL'INPUT VIENE EFFETTUATO, DI FATTO, QUI!
            --------------------------------------------------------------------------------
            */
            if (str == null)
                return;
            char[] num = str.toCharArray();
            // è stata inserita una cifra? 
            for (int i = 0; i < num.length; i++) {
                if (!(Character.isDigit(num[i])))
                    return;
            }
            // quanti caratteri sono già presenti?
            if (this.campoContaCaratteri.quantiCaratteri() >= NonPiuDiDueCifre.MAX_CHARS_ADMITTED)
                return;

            // se sono già state inserite fino a 2 cifre, è tutto ok
            //	e si prosegue secondo quanto avverrebbe di default 
            super.insertString(offs, str, a);
            return;
        }
    }
}