package motore;

import grafica.QuartoPassoPanel;
import java.util.Vector;
import java.io.*;
import java.util.StringTokenizer;
import struttura.ArrayOrdinatoCircolare;
import struttura.Corsia;
import struttura.Flusso;
import struttura.Incrocio;
import struttura.ListaFlussi;
import struttura.Strada;
import struttura.TipoTraffico;

public class JavaToNuSMV {

	private static JavaToNuSMV self;
        private Incrocio incrocio;
	private ArrayOrdinatoCircolare[] sorgenti;	// contiene le corsie entranti nell'incrocio
	private ArrayOrdinatoCircolare[] destinazioni;	// contiene le corsie uscenti dall'incrocio
	private ArrayOrdinatoCircolare strade;
	private ArrayOrdinatoCircolare attraversamenti;
	private ListaFlussi listaFlussi;
	private ListaFlussi flussiPedonali;
	private ListaFlussi flussiCappi;	// contiene quei flussi che hanno come sorgente e destinazione due corsie appartenenti alla stessa strada
	private ListaFlussi flussiVietati;
	private boolean concorrenzaStessaDestinazione;
	private int[] numCollisioni;
	private LauncherManager launcher;
        private String path = "C:\\Programmi\\NuSMV\\2.4.3\\bin\\NuSMV.exe";
        private int seconds = 60;
        private int bmc = 40;
        private boolean symbolic = true;
        private QuartoPassoPanel outputPanel;

	private JavaToNuSMV() {
		incrocio = Incrocio.getInstance();
		sorgenti = new ArrayOrdinatoCircolare[TipoTraffico.NUM_TIPI];
		destinazioni = new ArrayOrdinatoCircolare[TipoTraffico.NUM_TIPI];
		strade = new ArrayOrdinatoCircolare();
		attraversamenti = new ArrayOrdinatoCircolare();
		flussiPedonali = new ListaFlussi();
		flussiCappi = new ListaFlussi();
                flussiVietati = new ListaFlussi();
		concorrenzaStessaDestinazione = false;
		launcher = LauncherManager.getInstance();
	}
        
        public static JavaToNuSMV getInstance() {
            if (self == null)
                self = new JavaToNuSMV();
            return self;
        }
        
        public static JavaToNuSMV resetInstance() {
            self = new JavaToNuSMV();
            return self;
        }
        
        public void setPath(String aPath) {
            path = aPath;
        }
        
        public void setBmc(int aValue){
            bmc = aValue;
        }
        
        public void setSeconds(int aValue){
            seconds = aValue;
        }
        
        public void setSymbolic(boolean aValue){
            symbolic = aValue;
        }
        
        public void setOutputPanel(QuartoPassoPanel aPanel){
            outputPanel = aPanel;
        }
	
	public void setConcorrenzaStessaDestinazione(boolean valore) {
		concorrenzaStessaDestinazione = valore;
	}
        
        public void addFlussoVietato(String sorgente, String destinazione){
            int da = Integer.parseInt(sorgente);
            int a = Integer.parseInt(destinazione);
            flussiVietati.add(da, a);
        }
        
        public void deleteFlussoVietato(String sorgente, String destinazione){
            int da = Integer.parseInt(sorgente);
            int a = Integer.parseInt(destinazione);
            flussiVietati.removeFlusso(new Flusso(da, a));
        }
        
        public ListaFlussi getListaFlussiVietati(){
            return this.flussiVietati;
        }
	
	public void numerazione() {
		for (int i = 0; i < TipoTraffico.NUM_TIPI; i++) {
			sorgenti[i] = new ArrayOrdinatoCircolare();
			destinazioni[i] = new ArrayOrdinatoCircolare();
		}
		int counter = 0;
		Vector<Strada> listaStrade = incrocio.getListaStrade();
		for (int i = 0; i < listaStrade.size(); i++) {
			Strada strada = listaStrade.get(i);
			strada.setIndice(i);
			strade.add(i);
			Vector<Corsia> listaCorsieEntranti = strada.getListaCorsieEntranti();
			for (int j = 0; j < listaCorsieEntranti.size(); j++) {
				Corsia corsia = listaCorsieEntranti.get(j);
				corsia.setIndice(counter);
				Vector<Integer> tipiTraffico = corsia.getTipiTraffico();
				for (int k = 0; k < tipiTraffico.size(); k++) {
					sorgenti[tipiTraffico.get(k)].add(counter);
				}
				counter++;
			}
			Vector<Corsia> listaCorsieUscenti = strada.getListaCorsieUscenti();
			for (int j = 0; j < listaCorsieUscenti.size(); j++) {
				Corsia corsia = listaCorsieUscenti.get(j);
				corsia.setIndice(counter);
				Vector<Integer> tipiTraffico = corsia.getTipiTraffico();
				for (int k = 0; k < tipiTraffico.size(); k++) {
					destinazioni[tipiTraffico.get(k)].add(counter);
				}
				for (int k = 0; k < listaCorsieEntranti.size(); k++) {
					flussiCappi.add(listaCorsieEntranti.get(k).getIndice(), counter);
				}
				counter++;
			}
			Strada stradaSucc;
			if (i+1 < listaStrade.size())
				stradaSucc = listaStrade.get(i+1);
			else
				stradaSucc = listaStrade.get(0);
			if (strada.getAttraversamento() || stradaSucc.getAttraversamento()) {
				attraversamenti.add(counter);
				counter++;
			}
		}
		int j = 0;
		for (int i = 0; i < listaStrade.size(); i++) {
			Strada strada = listaStrade.get(i);
			if (strada.getAttraversamento()) {
				Flusso flussoPedonale;
				if (i == 0)
					flussoPedonale = new Flusso(attraversamenti.get(attraversamenti.size()-1), attraversamenti.get(j), TipoTraffico.PEDONI);
				else
					flussoPedonale = new Flusso(attraversamenti.get(j-1), attraversamenti.get(j), TipoTraffico.PEDONI);
				flussiPedonali.addFlusso(flussoPedonale);
				j++;
			}
			else if (i == 0 || (i > 0 && listaStrade.get(i-1).getAttraversamento()))
				j++;
		}
	}
	
	public void creaListaFlussi() {
		listaFlussi = new ListaFlussi();
		for (int tipoTraffico = 0; tipoTraffico < TipoTraffico.NUM_TIPI; tipoTraffico++) {
			for (int i = 0; i < sorgenti[tipoTraffico].size(); i++) {
				int sorgente = sorgenti[tipoTraffico].get(i);
				for (int j = 0; j < destinazioni[tipoTraffico].size(); j++) {
					int destinazione = destinazioni[tipoTraffico].get(j);
					Flusso flusso = new Flusso(sorgente, destinazione, tipoTraffico);
					listaFlussi.addFlusso(flusso);
				}
			}
		}
		
		listaFlussi.subtractListaFlussi(flussiVietati);
		listaFlussi.subtractListaFlussi(flussiCappi);
		numCollisioni = new int[listaFlussi.size()];
		associaStradeAFlussi();
		listaFlussi.addListaFlussi(flussiPedonali);
	}
	
	public ListaFlussi getListaFlussi() {
		return listaFlussi;
	}
	
	// funzione che associa ad ogni flusso, le relative strade sorgente e destinazione
	private void associaStradeAFlussi() {
		Vector<Strada> listaStrade = incrocio.getListaStrade();
		for (int i = 0; i < listaFlussi.size(); i++) {
			Flusso flusso = listaFlussi.getFlusso(i);
			int corsiaSorgente = flusso.getSorgente();
			int corsiaDestinazione = flusso.getDestinazione();
			boolean sorgenteSettata = false;
			boolean destinazioneSettata = false;
			for (int j = 0; j < listaStrade.size(); j++) {
				Strada strada = listaStrade.get(j);
				Vector<Corsia> listaCorsieEntranti = strada.getListaCorsieEntranti();
				if (!sorgenteSettata)
					for (int k = 0; k < listaCorsieEntranti.size(); k++) {
						if (listaCorsieEntranti.get(k).getIndice() == corsiaSorgente) {
							flusso.setStradaSorgente(strada.getIndice());
							sorgenteSettata = true;
							break;
						}	
					}
				Vector<Corsia> listaCorsieUscenti = strada.getListaCorsieUscenti();
				if(!destinazioneSettata)
					for (int k = 0; k < listaCorsieUscenti.size(); k++) {
						if (listaCorsieUscenti.get(k).getIndice() == corsiaDestinazione) {
							flusso.setStradaDestinazione(strada.getIndice());
							destinazioneSettata = true;
							break;
						}	
					}
				if (sorgenteSettata && destinazioneSettata)
					break;
			}
		}
	}
	
	public void creaFileNuSMV() {
		ListaFlussi flussi = listaFlussi;
		String text = "";
		text += "MODULE main\n";
		// sezione VAR
		text += "VAR\n";
		for (int i = 0; i < flussi.size(); i++) {
			Flusso flusso = flussi.getFlusso(i);
			text += "\t" + flussi.getFlussoVarName(flusso) + ": boolean; -- " + flusso + "\n";
		}
		text += "\n";
		// sezione LTLSPEC
		text += "LTLSPEC\n";
		text += "\t!(\n";
		text += "\t\t-- Formule che indicano che prima o poi un semaforo deve diventare verde\n";
		for (int i = 0; i < flussi.size(); i++) {
			Flusso flusso = flussi.getFlusso(i);
			text += "\t\tG F " + flussi.getFlussoVarName(flusso) + " &\n";
		}
		text += "\n";
		text += "\t\t-- Formule che evitano le collisioni\n";
		ArrayOrdinatoCircolare sources = new ArrayOrdinatoCircolare();
		ArrayOrdinatoCircolare destinations = new ArrayOrdinatoCircolare();
		for (int i = 0; i < TipoTraffico.NUM_TIPI; i++) {
			sources.union(sorgenti[i]);
			destinations.union(destinazioni[i]);
		}
                
                String textCollisioni = "";
		for (int i = 0; i < flussi.size(); i++) {
			Flusso flusso = flussi.getFlusso(i);
			Vector<Integer> leftSources = sources.getSubset(flusso.getSorgente(), flusso.getDestinazione());
			Vector<Integer> rightSources = sources.getSubset(flusso.getDestinazione(), flusso.getSorgente());
			Vector<Integer> leftDestinations = destinations.getSubset(flusso.getSorgente(), flusso.getDestinazione());
			Vector<Integer> rightDestinations = destinations.getSubset(flusso.getDestinazione(), flusso.getSorgente());
			ListaFlussi flussiConcorrenti = new ListaFlussi();
			for (int j = 0; j < leftSources.size(); j++) {
				int source = leftSources.get(j);
				if (source > flusso.getSorgente() || flussiPedonali.contains(flusso.getSorgente(), flusso.getDestinazione())) {
					if (flussi.contains(source, flusso.getDestinazione()) && concorrenzaStessaDestinazione)
						flussiConcorrenti.add(source, flusso.getDestinazione());
					for (int k = 0; k < rightDestinations.size(); k++)
						if (flussi.contains(source, rightDestinations.get(k)))
							flussiConcorrenti.add(source, rightDestinations.get(k));
				}
			}
			for (int j = 0; j < rightSources.size(); j++) {
				int source = rightSources.get(j);
				if (source > flusso.getSorgente() || flussiPedonali.contains(flusso.getSorgente(), flusso.getDestinazione())) {
					if (flussi.contains(source, flusso.getDestinazione()) && concorrenzaStessaDestinazione)
						flussiConcorrenti.add(source, flusso.getDestinazione());
					for (int k = 0; k < leftDestinations.size(); k++)
						if (flussi.contains(source, leftDestinations.get(k)))
							flussiConcorrenti.add(source, leftDestinations.get(k));
				}
			}
			if (!flussiConcorrenti.isEmpty()) {
                                if (!textCollisioni.equals(""))
                                    textCollisioni += " &\n";
				textCollisioni += "\t\tG (" + flussi.getFlussoVarName(flusso) + " -> ";
				for (int j = 0; j < flussiConcorrenti.size(); j++) {
					Flusso flussoConcorrente = flussiConcorrenti.getFlusso(j);
					if (j > 0)
						textCollisioni += " & ";
					textCollisioni += "!" + flussi.getFlussoVarName(flussoConcorrente);
				}
				textCollisioni += ")";
			}
		}
                text += textCollisioni;
                String textVerde = "";
		for (int i = 0; i < flussi.size(); i++) {
			Flusso flusso = flussi.getFlusso(i);
			if (flusso.getMinVerde() > 1) {
                                if (textVerde.equals("")) {
                                    text += " &\n\n";
                                    text += "\t\t-- Formule che esprimono la durata minima del verde\n";
                                }
                                else
                                    textVerde += " &\n";
 				textVerde += "\t\tG (!" + flussi.getFlussoVarName(flusso) + " & X " + flussi.getFlussoVarName(flusso) + " -> ";
				String prefix = "X X ";
				for (int j = 1; j < flusso.getMinVerde() - 1; j++) {
					textVerde += prefix + flussi.getFlussoVarName(flusso) + " & ";
					prefix += "X ";
				}
				textVerde += prefix + flussi.getFlussoVarName(flusso) + ")";
			}
		}
                text += textVerde;
		// Versione alternativa (leggermente più lenta)
		/*for (int i = 0; i < flussi.size(); i++) {
			Flusso flusso = flussi.getFlusso(i);
			
			for (int j = 1; j < flusso.getMinVerde(); j++) {
				if (j > 1)
				text += " &\n";
				text += "\t\tG!(!" + flussi.getFlussoVarName(flusso) + " & ";
				String prefix = "X ";
				for (int k = 0; k < j; k++) {
					text += prefix + flussi.getFlussoVarName(flusso) + " & ";
					prefix += "X ";
				}
				text += prefix + "!" + flussi.getFlussoVarName(flusso) + ")";
			}
			text += " &\n";
		}*/
                String textRosso = "";
		for (int i = 0; i < flussi.size(); i++) {
			Flusso flusso = flussi.getFlusso(i);
			if (flusso.getMaxRosso() > 0) {
				if (textRosso.equals("")) {
                                    text += " &\n\n";
                                    text += "\t\t-- Formule che esprimono la durata massima del rosso\n";
                                }
                                else
                                    textRosso += " &\n";
 				textRosso += "\t\tG (";
				String prefix = "";
				for (int j = 0; j < flusso.getMaxRosso(); j++) {
					if (j > 0)
						textRosso += " & ";
					textRosso += prefix + "!" + flussi.getFlussoVarName(flusso);
					prefix += "X ";
				}
				textRosso += " -> " + prefix + flussi.getFlussoVarName(flusso) + ")";
			}
		}
                text += textRosso;
		text += "\n";
		text += "\t)\n";
		
		FileOutputStream file;
		PrintStream output;
		try {
			file = new FileOutputStream("Incrocio.smv");
			output = new PrintStream(file);
			output.println(text);
			output.close();
		}
		catch (IOException e) {
			System.err.println("Errore: " + e);
			System.exit(1);
		}
                if (symbolic)
                    launcher.executeSymbolicNuSMV(path, "Incrocio.smv", seconds, outputPanel, "");
                else
                    launcher.executeBoundedNuSMV(path, "Incrocio.smv", bmc, seconds, outputPanel, "");
                launcher.startExecution();
                parser("Incrocio.smv");	// contiamo il numero di collissioni dei flussi
                sostituisciFlusso();
	}
	
	// funzione che parsa il file incrocio.smv
	// conta il numero di collisioni che ha ogni flusso
	public void parser(String nomeFile) {
		try {
			FileReader reader = new FileReader(nomeFile);
			BufferedReader file = new BufferedReader(reader);
			String line;
			boolean readingCollisionConstraints = false;
			while ((line = file.readLine()) != null) {
				if (readingCollisionConstraints) {
					if (((line.toLowerCase()).trim()).equals("") || ((line.toLowerCase()).trim()).equals(")")) {
						readingCollisionConstraints = false;
						break;
					}
					boolean first = true;
					int firstElem = 0;  // firstElem è il flusso a sinistra dell'implicazione
					StringTokenizer st = new StringTokenizer(line, "()");
					st.nextToken();
					String tok = st.nextToken();
					st = new StringTokenizer(tok, " ->f!&");
					while (st.hasMoreTokens()) {
						String elem = st.nextToken();
						if (!elem.equals("")) {
							if (first) {
								first = false;
								if ( (firstElem = Integer.parseInt(elem)) >= numCollisioni.length)
									break;
							}
							else {
								numCollisioni[firstElem]++;
								numCollisioni[Integer.parseInt(elem)]++;
							}
						}
					}
				}
				if (((line.toLowerCase()).trim()).equals("-- formule che evitano le collisioni"))
					readingCollisionConstraints = true;
			}
			file.close();
			reader.close();
		} catch(IOException e) {}
	}
	
	// funzione che modifica il file NuSMV (creandone uno nuovo), sostituendo il flusso f = f1 + f2
	public void modificaFileNuSMV(String nomeFile, String f, String f1, String f2) {
		String text = "";
		try {
			FileReader reader = new FileReader(nomeFile);
			BufferedReader file = new BufferedReader(reader);
			String line;
			boolean readingCollisionConstraints = false;
			while ((line = file.readLine()) != null) {
				if (((line.toLowerCase()).trim()).equals("")) {
					readingCollisionConstraints = false;
				}
				if (line.indexOf(f) != -1) {
					if (line.indexOf(": boolean;") != -1) {}
					else if (readingCollisionConstraints) {
						if ( (line.indexOf("(" + f + " ->") != -1) || (line.indexOf(" -> !" + f + ")") != -1) ) {}
						else {
							String tmp = line.replaceAll("!" + f + " & ", "");
							tmp = tmp.replaceAll(" & !" + f, "");
							text += tmp + "\n";
						}
					}
					else
						text += line.replaceAll(f, "(" + f1 + " & " + f2 + ")") + "\n";
				}
				else
					text += line + "\n";
				if (((line.toLowerCase()).trim()).equals("-- formule che evitano le collisioni"))
					readingCollisionConstraints = true;
			}
			file.close();
			reader.close();
		} catch(IOException e) {}
		
		FileOutputStream fileOut;
		PrintStream output;
		try {
			fileOut = new FileOutputStream(nomeFile.substring(0, nomeFile.length() - 4) + f + f1 + f2 + ".smv");
			output = new PrintStream(fileOut);
			output.println(text);
			output.close();
		}
		catch (IOException e) {
			System.err.println("Errore: " + e);
			System.exit(1);
		}
	}
	
	// BISOGNA GESTIRE IL FATTO CHE ALLA BOTTA SUCCESSIVA, SE HO SCELTO UN FLUSSO ORA, DOVRO' SCEGLIERE UN FLUSSO DIVERSO
	// questa funzione trova i due flussi che servono per sostituire il flusso col maggior numero di collisioni e chiama la funzione che crea il file NuSMV modificato
	public void sostituisciFlusso() {
		int round = 1;
		int indexFlusso;
		Vector<String[]> combinazioniFlussi = new Vector<String[]>(); // questo vettore contiene l'elenco ordinato in base al numero di collisioni di tutte le triple (f, f1, f2) dove f=f1+f2 ed f provoca almeno una collisione
		while ( (indexFlusso = maxCollisioni(round)) != -1) {
			Flusso flusso = listaFlussi.getFlusso(indexFlusso);
			int stradaSorgente = flusso.getStradaSorgente();
			int stradaDestinazione = flusso.getStradaDestinazione();
			Vector<Integer> stradeCandidate = strade.getSubset(stradaDestinazione, stradaSorgente);	// prendo tutte le strade comprese tra destinazione e sorgente
			Vector<Strada> listaStrade = incrocio.getListaStrade();
			
			// al termine del ciclo while, in stradeCandidate si avranno solo le strade che potranno essere utilizzate per sostituire il flusso che si vuole spezzare
			int i = 0;
			while (i < stradeCandidate.size()) {
				if (!listaStrade.get(stradeCandidate.get(i)).getInversione()) {
					stradeCandidate.remove(i);
				}
				else
					i++;
			}
			
			for (i = 0; i < stradeCandidate.size(); i++) {
				Strada stradaCandidata = listaStrade.get(stradeCandidate.get(i));
				int corsiaSorgente = flusso.getSorgente();
				int corsiaDestinazione = flusso.getDestinazione();
				int destinazioneIntermedia = -1;
				int sorgenteIntermedia = -1;
				Vector<Corsia> corsieUscenti = stradaCandidata.getListaCorsieUscenti();
				for (int j = 0; j < corsieUscenti.size(); j++) {
					if (corsieUscenti.get(j).getTipiTraffico().contains(flusso.getTipoTraffico()))
						destinazioneIntermedia = corsieUscenti.get(j).getIndice();
				}
				Vector<Corsia> corsieEntranti = stradaCandidata.getListaCorsieEntranti();
				for (int j = 0; j < corsieEntranti.size(); j++) {
					if (corsieEntranti.get(j).getTipiTraffico().contains(flusso.getTipoTraffico()))
						sorgenteIntermedia = corsieEntranti.get(j).getIndice();
				}
				Flusso f1 = new Flusso(corsiaSorgente, destinazioneIntermedia);
				Flusso f2 = new Flusso(sorgenteIntermedia, corsiaDestinazione);
				if (listaFlussi.contains(f1.getSorgente(), f1.getDestinazione()) && listaFlussi.contains(f2.getSorgente(), f2.getDestinazione())) {
					String descrizione = "Provo la sostituzione: " + listaFlussi.getFlussoVarName(flusso) + "(" + flusso.getSorgente() + "," + flusso.getDestinazione() + ") = " + listaFlussi.getFlussoVarName(f1) + "(" + f1.getSorgente() + "," + f1.getDestinazione() + ") + " + listaFlussi.getFlussoVarName(f2) + "(" + f2.getSorgente() + "," + f2.getDestinazione() + ")";
					modificaFileNuSMV("Incrocio.smv", listaFlussi.getFlussoVarName(flusso), listaFlussi.getFlussoVarName(f1), listaFlussi.getFlussoVarName(f2));
                                        if (symbolic)
                                            launcher.executeSymbolicNuSMV(path, "Incrocio" + listaFlussi.getFlussoVarName(flusso) + listaFlussi.getFlussoVarName(f1) + listaFlussi.getFlussoVarName(f2) + ".smv", seconds, outputPanel, descrizione);
                                        else
                                            launcher.executeBoundedNuSMV(path, "Incrocio" + listaFlussi.getFlussoVarName(flusso) + listaFlussi.getFlussoVarName(f1) + listaFlussi.getFlussoVarName(f2) + ".smv", bmc, seconds, outputPanel, descrizione);
					String[] tripla = new String[3];
					tripla[0] = listaFlussi.getFlussoVarName(flusso);
					tripla[1] = listaFlussi.getFlussoVarName(f1);
					tripla[2] = listaFlussi.getFlussoVarName(f2);
					combinazioniFlussi.add(tripla);
				}
			}
			round++;
		}
		
		for (int i = 0; i < combinazioniFlussi.size() - 1; i++) {
			String[] triplaUno = combinazioniFlussi.get(i);
			for (int j = i + 1; j < combinazioniFlussi.size(); j++) {
				String[] triplaDue = combinazioniFlussi.get(j);
				Vector<String[]> triple = new Vector<String[]>();
				triple.add(triplaUno);
				triple.add(triplaDue);
				if ( !(triplaUno[0].equals(triplaDue[0])) && (verificaCompatibilita(triple)) ) {
                                        Flusso fl11 = listaFlussi.getFlussoByVarName(triplaUno[0]);
					Flusso fl12 = listaFlussi.getFlussoByVarName(triplaUno[1]);
					Flusso fl13 = listaFlussi.getFlussoByVarName(triplaUno[2]);
					Flusso fl21 = listaFlussi.getFlussoByVarName(triplaDue[0]);
					Flusso fl22 = listaFlussi.getFlussoByVarName(triplaDue[1]);
					Flusso fl23 = listaFlussi.getFlussoByVarName(triplaDue[2]);
					String descrizione = "Provo la doppia sostituzione: " + triplaUno[0] + "(" + fl11.getSorgente() + "," + fl11.getDestinazione() + ") = " + triplaUno[1]  + "(" + fl12.getSorgente() + "," + fl12.getDestinazione() + ") + " + triplaUno[2]  + "(" + fl13.getSorgente() + "," + fl13.getDestinazione() + ") e " + triplaDue[0]  + "(" + fl21.getSorgente() + "," + fl21.getDestinazione() + ") = " + triplaDue[1]  + "(" + fl22.getSorgente() + "," + fl22.getDestinazione() + ") + " + triplaDue[2]  + "(" + fl23.getSorgente() + "," + fl23.getDestinazione() + ")";
					modificaFileNuSMV("Incrocio" + triplaUno[0] + triplaUno[1] + triplaUno[2] + ".smv", triplaDue[0], triplaDue[1], triplaDue[2]);
					if (symbolic)
                                            launcher.executeSymbolicNuSMV(path, "Incrocio" + triplaUno[0] + triplaUno[1] + triplaUno[2] + triplaDue[0] + triplaDue[1] + triplaDue[2] + ".smv", seconds, outputPanel, descrizione);
                                        else
                                            launcher.executeBoundedNuSMV(path, "Incrocio" + triplaUno[0] + triplaUno[1] + triplaUno[2] + triplaDue[0] + triplaDue[1] + triplaDue[2] + ".smv", bmc, seconds, outputPanel, descrizione);
                                }
			}
		}
	}
	
	// funzione che verifica la compatibilità di flussi spezzati simultaneamente
	public boolean verificaCompatibilita(Vector<String[]> listaFlussi) {
		Vector<String> flussiSpezzati = new Vector<String>();
		Vector<String> flussiSostitutivi = new Vector<String>();
		for (int i = 0; i < listaFlussi.size(); i++) {
			String[] tripla = listaFlussi.get(i);
			flussiSpezzati.add(tripla[0]);
			flussiSostitutivi.add(tripla[1]);
			flussiSostitutivi.add(tripla[2]);
		}
		for (int i = 0; i < flussiSpezzati.size(); i++) {
			if (flussiSostitutivi.contains(flussiSpezzati.get(i)))
				return false;
		}
		return true;
	}
	
	// funzione che ritorna il 'round-esimo' flusso che provoca il maggior numero di collisioni
	// oppure ritorna -1 se il 'round-esimo' flusso ha 0 collisioni
	public int maxCollisioni(int round) {
		int max = 0;
		int maxIndice = -1;
		boolean[] collisioniBool = new boolean[numCollisioni.length];
		
		for (int k = 0; k < collisioniBool.length; k++)
			collisioniBool[k] = false;
		
		for (int j = 0; j < round; j++) {
			max = 0;
			maxIndice = -1;
			for (int i = 0; i < numCollisioni.length; i++)
				if ( (numCollisioni[i] > max) && (!collisioniBool[i]) ) {
					max = numCollisioni[i];
					maxIndice = i;
				}
			if (maxIndice != -1)
				collisioniBool[maxIndice] = true;
		}
		return maxIndice;
	}
	
}