package motore;

import grafica.QuartoPassoPanel;
import java.util.Vector;

public class LauncherManager implements Runnable {
    
    private static LauncherManager self = null;
    private Vector<Vector> commands;
    private boolean running = false;
    private Process currentProcess = null;
    
    private LauncherManager() {
        commands = new Vector<Vector>();
    }
    
    public static LauncherManager getInstance() {
        if (self == null)
            self = new LauncherManager();
	return self;
    }
    
    public static LauncherManager resetInstance() {
        self = new LauncherManager();
	return self;
    }

    public void executeSymbolicNuSMV(String pathNuSMV, String filename, long seconds, QuartoPassoPanel outputPanel, String descrizione) {
        Vector command = new Vector();
        command.add(pathNuSMV + " " + filename);
        command.add(seconds);
        command.add(outputPanel);
        command.add(descrizione);
        commands.add(command);
    }
    
    public void executeBoundedNuSMV(String pathNuSMV, String filename, int bmcLength, long seconds, QuartoPassoPanel outputPanel, String descrizione) {
        Vector command = new Vector();
        command.add(pathNuSMV + " -bmc -bmc_length " + bmcLength + " " + filename);
        command.add(seconds);
        command.add(outputPanel);
        command.add(descrizione);
        commands.add(command);
    }
    
    public synchronized void setCurrentProcess(Process process) {
        currentProcess = process;
    }
    
    public synchronized void startExecution() {
        if (!isRunning()) {
            running = true;
            (new Thread(this)).start();
        }
    }
    
    public synchronized void stopExecution() {
        running = false;
        if (currentProcess != null)
            currentProcess.destroy();
        commands = new Vector<Vector>();
    }
    
    public synchronized boolean isRunning() {
        return running;
    }
    
    public void run() {
        while (isRunning()) {
            if (!commands.isEmpty()) {
                Vector command = commands.get(0);
                String commandString = (String) command.get(0);
                long seconds = (Long) command.get(1);
                QuartoPassoPanel outputPanel = (QuartoPassoPanel) command.get(2);
                String descrizione = (String) command.get(3);                
                try {
                    Launcher launcher = new Launcher(commandString, seconds, outputPanel, descrizione);
                    Thread thread = new Thread(launcher); 
                    thread.start();
                    thread.join();
                }
                catch (Exception e) {
                }                
                if (!commands.isEmpty())
                    commands.removeElementAt(0);
            }
        }
    }
    
}