package struttura;

import java.io.Serializable;

public class Flusso implements Serializable {
	
	private int sorgente;	// è l'indice della corsia sorgente del flusso
	private int destinazione;	// è l'indice della corsia destinazione del flusso
	private int stradaSorgente;	// è l'indice della strada sorgente del flusso; vale -1 per un flusso pedonale
	private int stradaDestinazione;	// è l'indice della strada destinazione del flusso; vale -1 per un flusso pedonale
	private int minVerde;	// è la durata minima del verde per il semaforo di questo flusso in istanti di tempo
	private int maxRosso;	// è la durata massima del rosso per il semaforo di questo flusso in istanti di tempo
	private int tipoTraffico;	// è la tipologia di traffico per questo flusso (un flusso ha una ed una sola tipologia di traffico)
	
	// Costruttore che prende in input due parametri: corsia sorgente e destinazione
	public Flusso(int aSorg, int aDest) {
		sorgente = aSorg;
		destinazione = aDest;
		minVerde = -1;
		maxRosso = -1;
	}
	
	// Costruttore che prende in input tre parametri: corsia sorgente e destinazione e tipo di traffico della corsia
	public Flusso(int aSorg, int aDest, int aTipo) {
		sorgente = aSorg;
		destinazione = aDest;
		tipoTraffico = aTipo;
		minVerde = -1;
		maxRosso = -1;
		
		if (tipoTraffico == TipoTraffico.PEDONI) {
			stradaSorgente = -1;
			stradaDestinazione = -1;
		}
	}
	
	
	// METODI DI SET DELLE PROPRIETA'
	
	public void setStradaSorgente(int sorgente) {
		stradaSorgente = sorgente;
	}
	
	public void setStradaDestinazione(int destinazione) {
		stradaDestinazione = destinazione;
	}
	
	public void setMinVerde(int durata) {
		minVerde = durata;
	}
	
	public void setMaxRosso(int durata) {
		maxRosso = durata;
	}
	
	
	// METODI DI GET DELLE PROPRIETA'
	
	public int getSorgente() {
		return sorgente;
	}
	
	public int getDestinazione() {
		return destinazione;
	}
	
	public int getStradaSorgente() {
		return stradaSorgente;
	}
	
	public int getStradaDestinazione() {
		return stradaDestinazione;
	}
	
	public int getTipoTraffico() {
		return tipoTraffico;
	}
	
	public int getMinVerde() {
		return minVerde;
	}
	
	public int getMaxRosso() {
		return maxRosso;
	}
	
	
	// ALTRI METODI
	
	public String toString() {
		return "(" + sorgente + ", " + destinazione + ")";
	}
	
	public boolean equals(Object obj) {
		Flusso flusso = (Flusso)obj;
		if (sorgente == flusso.getSorgente() && destinazione == flusso.getDestinazione())
			return true;
		return false;
	}
	
}